#!/usr/bin/env python

"""
bayesian inversion

 Solve the equation
  shat = sprior + HQT*inverse(HQHT+R)*(z-hsp)

Input files:
	HQHT.npy, HQHT_pbl.npy, HQHT_trans.npy, HQHT_ftrop.npy
	r.txt
	zhsp.txt
	sprior.npy
	All HQ slice files

Output files:
	hqhti.npy
	hqht_zhsp.npy
	qhts.npy
	hshat.txt
	shat_flux.npy, shat_flux.nc
	shat_pbl.npy, shat_pbl.nc
	shat_trans.npy, shat_trans.nc
	shat_ftrop.npy, shat_ftrop.nc

No boundary value optimization

"""

from __future__ import print_function

import sys
import optparse
import datetime
import numpy

import utils


####################################################################
def qht_s(config, hqht_zhsp):
	""" compute hqt*[inverse(hqht+r)*(z-h*sp)]

	inverse(hqht+r)*(z-h*sp) is given as the array hqht_zhsp.
	hq is read in from files created previously.
	"""

	ntimesteps = config["ntimesteps"]

	qhts = numpy.zeros((ntimesteps, config["locations"]))

	t0 = datetime.datetime.now()
	for i in range(ntimesteps):
		t1 = datetime.datetime.now()

		# hq slice files are numbered 1 .. ntimesteps
		# hq shape is #obs x #cells
		hqfile = config["workdir"] + "/" + config["hqdir"] + "/HQ%04d.npy" % (i+1)
		hq = numpy.load(hqfile)

		qhts[i] = numpy.dot(hq.T, hqht_zhsp)

		t2a = datetime.datetime.now()
		print("Step %4d" % (i+1), "of ", ntimesteps, "done. Step time ", t2a-t1, "Total elapsed time ", t2a-t0)

	return qhts


####################################################################
if __name__ == '__main__':

	numpy.set_printoptions(precision=6, edgeitems=10, linewidth=150)

	parser = optparse.OptionParser(usage='%prog [options] receptor_file', description="Do bayesian inversion. ")
	parser.add_option('-c', '--config', default="config.ini", help="set configuration file to use.  Default is 'config.ini'")
	parser.add_option('-r', '--resume', action="store_true", default=False, help="resume calculations at the qhts step")

	options, args = parser.parse_args()

	if len(args) != 2:
		parser.print_usage()
		sys.exit()

	spriorfile = args[0]
	rfile = args[1]

	config = utils.getConfig(options.config)

	tstart = datetime.datetime.now()


	# Get hqht calculated previously.
	# shape of hqht is numobs x numobs
	print("Getting hqht...")
	filename = config["workdir"] + "/" + config["hqdir"] + "/HQHT.npy"
	hqflux = numpy.load(filename)
	numobs = hqflux.shape[0]
	config["num_obs"] = numobs

	hqht = hqflux


	print("Adding r to hqht...")
	r = numpy.loadtxt(rfile)
	for i in range(numobs):
		hqht[i, i] = hqht[i, i] + r[i]


	# calculate inverse of hqht
	# hqhti is needed later in aposterior variance calculations
	if options.resume:
		print("Loading hqhti...")
		hqhti = numpy.load("hqhti.npy")
	else:
		print("Calculate inverse of hqht...")
		hqhti = numpy.linalg.inv(hqht)
		numpy.save("hqhti", hqhti)


	print("Getting zhsp...")
	zfile = config["workdir"] + "/" + config["z_file"]
	zhsp = numpy.loadtxt(zfile)

	if options.resume:
		print("Loading hqht_zhs...")
		hqht_zhsp = numpy.load("hqht_zhsp.npy")
	else:
		print("Calculating hqhti*zhsp...")
		hqht_zhsp = numpy.dot(hqhti, zhsp)
		numpy.save("hqht_zhsp", hqht_zhsp)

	# compute qht*[inverse(hqht+r)*(z-h*sp)]
	print("Calculating qhts...")
	qhts = qht_s(config, hqht_zhsp)
	print("qhts shape is ", qhts.shape)
	numpy.save("qhts", qhts)

	print("Getting sprior...")
	try:
		sp = numpy.load(spriorfile)
	except:
		try:
			landmask = numpy.load(config["landmask_file"])
			sp = utils.getSpriorNetcdf(spriorfile, "sprior", landmask)
		except:
			print("Can not read sprior file.")
			sys.exit()


	print("sp shape is ", sp.shape)


	print("Calculate shat...")
	shat = sp + qhts

	print("Saving shat in netcdf format...")
	numpy.save("shat", shat)
	utils.make_ncdf("shat.nc", "shat", shat, config)


	print("Making hshat...")
	hshat = utils.convolve(shat, config)
	numpy.savetxt("hshat.txt", hshat)


	tdone = datetime.datetime.now()
	print("Finished. Elapsed time", tdone-tstart)
